************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
************************************************************************
      Subroutine Get_PC_Coord_New(Coord,nData)
      Implicit Real*8 (a-h,o-z)
#include "stdalloc.fh"

      Character*24 Label
      Logical      Found
      Real*8, Dimension(:), Allocatable :: Coord

      Label='GeoNewPC'
      Call qpg_dArray(Label,Found,nData)
      If(.not.Found .or. nData.eq.0) Return
      call mma_allocate(Coord,nData,label='Coord')
      Call Get_dArray(Label,Coord,nData)

      Return
      End
