!***********************************************************************
! This file is part of OpenMolcas.                                     *
!                                                                      *
! OpenMolcas is free software; you can redistribute it and/or modify   *
! it under the terms of the GNU Lesser General Public License, v. 2.1. *
! OpenMolcas is distributed in the hope that it will be useful, but it *
! is provided "as is" and without any express or implied warranties.   *
! For more details see the full text of the license in the file        *
! LICENSE or in <http://www.gnu.org/licenses/>.                        *
!***********************************************************************

subroutine Assemble_mGauss(As,Ad,nAs)

use Definitions, only: wp, iwp

implicit none
integer(kind=iwp), intent(in) :: nAs
real(kind=wp), intent(inout) :: As(nAs)
real(kind=wp), intent(in) :: Ad(nAs,6)

As(:) = As+Ad(:,1)+Ad(:,4)+Ad(:,6)

return

end subroutine Assemble_mGauss
