!***********************************************************************
! This file is part of OpenMolcas.                                     *
!                                                                      *
! OpenMolcas is free software; you can redistribute it and/or modify   *
! it under the terms of the GNU Lesser General Public License, v. 2.1. *
! OpenMolcas is distributed in the hope that it will be useful, but it *
! is provided "as is" and without any express or implied warranties.   *
! For more details see the full text of the license in the file        *
! LICENSE or in <http://www.gnu.org/licenses/>.                        *
!                                                                      *
! Copyright (C) 2017, Stefan Knecht                                    *
!***********************************************************************

module active_space_solver_cfg

use Definitions, only: iwp

implicit none
private

character(len=8) :: as_solver = '        '
logical(kind=iwp) :: as_solver_inp_proc = .false.

public :: as_solver, as_solver_inp_proc

end module active_space_solver_cfg
