!***********************************************************************
! This file is part of OpenMolcas.                                     *
!                                                                      *
! OpenMolcas is free software; you can redistribute it and/or modify   *
! it under the terms of the GNU Lesser General Public License, v. 2.1. *
! OpenMolcas is distributed in the hope that it will be useful, but it *
! is provided "as is" and without any express or implied warranties.   *
! For more details see the full text of the license in the file        *
! LICENSE or in <http://www.gnu.org/licenses/>.                        *
!                                                                      *
! Copyright (C) 2020, Roland Lindh                                     *
!***********************************************************************

subroutine Alaska_banner()

use Definitions, only: u6

implicit none

! figlet -f larry3d.flf Alaska
write(u6,100) " ______   ___                     __"
write(u6,100) "/\  _  \ /\_ \                   /\ \"
write(u6,100) "\ \ \L\ \\//\ \      __      ____\ \ \/'\      __"
write(u6,100) " \ \  __ \ \ \ \   /'__`\   /',__\\ \ , <    /'__`\"
write(u6,100) "  \ \ \/\ \ \_\ \_/\ \L\.\_/\__, `\\ \ \\`\ /\ \L\.\_"
write(u6,100) "   \ \_\ \_\/\____\ \__/.\_\/\____/ \ \_\ \_\ \__/.\_\"
write(u6,100) "    \/_/\/_/\/____/\/__/\/_/\/___/   \/_/\/_/\/__/\/_/"
write(u6,100) ""
write(u6,100) ""

100 format(25x,a)

end subroutine Alaska_banner
