#***********************************************************************
# This file is part of OpenMolcas.                                     *
#                                                                      *
# OpenMolcas is free software; you can redistribute it and/or modify   *
# it under the terms of the GNU Lesser General Public License, v. 2.1. *
# OpenMolcas is distributed in the hope that it will be useful, but it *
# is provided "as is" and without any express or implied warranties.   *
# For more details see the full text of the license in the file        *
# LICENSE or in <http://www.gnu.org/licenses/>.                        *
#                                                                      *
# Copyright (C) 2020, Oskar Weser                                      *
#***********************************************************************

#  """
#  Entrypoint module, in case you use `python -manalyze_molcas`.
#
#
#  Why does this file exist, and why __main__? For more info, read:
#
#  - https://www.python.org/dev/peps/pep-0338/
#  - https://docs.python.org/2/using/cmdline.html#cmdoption-m
#  - https://docs.python.org/3/using/cmdline.html#cmdoption-m
#  """
#  from analyze_molcas.cli import main
#
#  if __name__ == "__main__":
#      main()
